package com.devassistant.file.service;

import com.devassistant.file.model.FileRecord;
import com.devassistant.file.repository.FileRecordRepository;
import com.devassistant.util.IpUtil;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import lombok.RequiredArgsConstructor;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.domain.Sort;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.io.*;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.Date;
import java.util.List;

@Slf4j
@Service
@RequiredArgsConstructor
public class FileService {

    private final FileRecordRepository fileRecordRepository;
    @Value("${file.upload-dir}")
    private String uploadDir;

    @SneakyThrows
    public void uploadFile(MultipartFile file, HttpServletRequest request) {
        String originalFilename = file.getOriginalFilename();
        if (originalFilename == null) {
            throw new RuntimeException("文件名为空");
        }

        // 创建上传目录（如果不存在）
        Path uploadPath = Paths.get(uploadDir);
        if (!Files.exists(uploadPath)) {
            Files.createDirectories(uploadPath);
        }

        Path filePath = uploadPath.resolve(originalFilename);

        // 如果文件已存在，则递增文件名
        int i = 1;
        String fileNameWithoutExtension = getFileNameWithoutExtension(originalFilename);
        String fileExtension = getFileExtension(originalFilename);

        while (Files.exists(filePath)) {
            // 生成新的文件名（加上_1, _2 等）
            String newFileName = fileNameWithoutExtension + "_" + i + fileExtension;
            filePath = uploadPath.resolve(newFileName);
            i++;
        }

        // 复制文件
        Files.copy(file.getInputStream(), filePath);

        FileRecord fileRecord = new FileRecord();
        fileRecord.setUploadIp(IpUtil.getIpAddr(request));
        fileRecord.setFileName(filePath.getFileName().toString());
        fileRecord.setFileSize(Files.size(filePath));
        fileRecord.setFilePath(filePath.toString());
        fileRecord.setFileType(fileExtension);
        fileRecord.setUploadTime(new Date());
        fileRecordRepository.save(fileRecord);
    }

    public List<FileRecord> getFileList() {
        Sort sort = Sort.by(Sort.Direction.DESC, "uploadTime");
        return fileRecordRepository.findAll(sort);
    }

    // 获取文件名（去除扩展名）
    private String getFileNameWithoutExtension(String filename) {
        int dotIndex = filename.lastIndexOf(".");
        return (dotIndex == -1) ? filename : filename.substring(0, dotIndex);
    }

    // 获取文件扩展名
    private String getFileExtension(String filename) {
        int dotIndex = filename.lastIndexOf(".");
        return (dotIndex == -1) ? "" : filename.substring(dotIndex);
    }

    /**
     * 删除文件
     *
     * @param id 文件id
     */
    @SneakyThrows
    public void delFile(Integer id) {
        FileRecord fileRecord = fileRecordRepository.findById(id).orElse(null);
        if (fileRecord != null) {
            Path filePath = Paths.get(fileRecord.getFilePath());
            Files.delete(filePath);
        }
        fileRecordRepository.deleteById(id);
    }

    /**
     * 文件下载
     *
     * @param id       文件id
     * @param response 响应
     */
    public void downloadFile(Integer id, HttpServletResponse response) {
        FileRecord fileRecord = fileRecordRepository.findById(id).orElseThrow();
        Path filePath = Paths.get(fileRecord.getFilePath());
        File file = filePath.toFile();

        if (!file.exists()) {
            throw new RuntimeException("文件不存在");
        }

        // 设置响应头
        response.setContentType("application/octet-stream");
        response.setHeader("Content-Disposition", "attachment; filename=\"" + URLEncoder.encode(file.getName(), StandardCharsets.UTF_8) + "\"");
        response.setContentLengthLong(file.length());

        try (InputStream inputStream = new FileInputStream(file);
             OutputStream outputStream = response.getOutputStream()) {

            // 文件流拷贝
            byte[] buffer = new byte[8192];
            int bytesRead;
            while ((bytesRead = inputStream.read(buffer)) != -1) {
                outputStream.write(buffer, 0, bytesRead);
            }

            outputStream.flush();
        } catch (IOException e) {
            throw new RuntimeException("文件下载失败", e);
        }
    }
}
